
-- The "modified attribute" takes the form of: modifier-name-button
-- The modifier is one of "shift-", "ctrl-", "alt-", and the button is a number from 1 through 5.
--
-- For example, you could set an action that is used for unmodified left click like this:
-- self:SetAttribute("action1", value);
-- You could set an action that is used for shift-right-click like this:
-- self:SetAttribute("shift-action2", value);
--
-- You can also use a wildcard in the place of the modifier or button to denote an attribute
-- that is used for any modifier or any button.
--
-- For example, you could set an action that is used for any left click like this:
-- self:SetAttribute("*action1", value);
-- You could set an action that is used for shift-click with any button like this:
-- self:SetAttribute("shift-action*", value);
--
ATTRIBUTE_NOOP = "";
-- You can exclude an action by explicitly setting its value to ATTRIBUTE_NOOP
--
-- For example, you could set an action that was used for all clicks except ctrl-left-click:
-- self:SetAttribute("*action*", value);
-- self:SetAttribute("shift-action1", ATTRIBUTE_NOOP);
--
-- Setting the attribute by itself is equivalent to *attribute*

function SecureButton_GetModifierPrefix()
    local prefix = "";
    if ( IsShiftKeyDown() ) then
        prefix = "shift-"..prefix;
    end
    if ( IsControlKeyDown() ) then
        prefix = "ctrl-"..prefix;
    end
    if ( IsAltKeyDown() ) then
        prefix = "alt-"..prefix;
    end
	return prefix;
end

function SecureButton_GetButtonSuffix(button)
	local suffix = "";
    if ( button == "LeftButton" ) then
        suffix = "1";
    elseif ( button == "RightButton" ) then
        suffix = "2";
    elseif ( button == "MiddleButton" ) then
        suffix = "3";
    elseif ( button == "Button4" ) then
        suffix = "4";
    elseif ( button == "Button5" ) then
        return "5";
    elseif ( button and button ~= "" ) then
        suffix = "-" .. tostring(button);
    end
    return suffix;
end

function SecureButton_GetModifiedAttribute(frame, name, button)
    local modifier = SecureButton_GetModifierPrefix();
    local suffix = SecureButton_GetButtonSuffix(button);
    local value = frame:GetAttribute(modifier..name..suffix) or
        frame:GetAttribute("*"..name..suffix) or
        frame:GetAttribute(modifier..name.."*") or
        frame:GetAttribute("*"..name.."*") or
        frame:GetAttribute(name);
    if ( not value and frame:GetAttribute("parentAttribute") ) then
        value = SecureButton_GetModifiedAttribute(frame:GetParent(), name, button);
    end
    if ( value == ATTRIBUTE_NOOP ) then
        value = nil;
    end
    return value;
end

-- SecureActionButtons allow you to map different combinations of modifiers and buttons into
-- actions which are executed when the button is clicked.
--
-- For example, you could set up the button to respond to left clicks by targeting the focus:
-- self:SetAttribute("unit", "focus");
-- self:SetAttribute("type1", "target");
--
-- You could set up all other buttons to bring up a menu like this:
-- self:SetAttribute("type*", "menu");
-- self.showmenu = menufunc;
--
-- SecureActionButtons are also able to perform different actions depending on whether you can
-- attack the unit or assist the unit associated with the action button.  It does so by mapping
-- mouse buttons into "virtual buttons" based on the state of the unit. For example, you can use
-- the following to cast "Mind Blast" on a left click and "Shadow Word: Death" on a right click
-- if the unit can be attacked:
-- self:SetAttribute("harmbutton1", "nuke1");
-- self:SetAttribute("type-nuke1", "spell");
-- self:SetAttribute("spell-nuke1", "Mind Blast");
-- self:SetAttribute("harmbutton2", "nuke2");
-- self:SetAttribute("type-nuke2", "spell");
-- self:SetAttribute("spell-nuke2", "Shadow Word: Death");
--
-- In this example, we use the special attribute "harmbutton" which is used to map a virtual
-- button when the unit is attackable. We also have the attribute "helpbutton" which is used
-- when the unit can be assisted.
--
-- Although it may not be immediately obvious, we are able to use this new virtual button
-- to set up very complex click behaviors on buttons. For example, we can define a new "heal"
-- virtual button for all friendly left clicks, and then set the button to cast "Flash Heal"
-- on an unmodified left click and "Renew" on a ctrl left click:
-- self:SetAttribute("*helpbutton1", "heal");
-- self:SetAttribute("*type-heal", "spell");
-- self:SetAttribute("spell-heal", "Flash Heal");
-- self:SetAttribute("ctrl-spell-heal", "Renew");
--
-- This system is very powerful, and provides a good layer of abstraction for setting up
-- a button's click behaviors.

function SecureActionButton_OnClick(self, button)

	-- Lookup the unit, based on the modifiers and button
	local unit = SecureButton_GetModifiedAttribute(self, "unit", button);
    local unitSuffix = SecureButton_GetModifiedAttribute(self, "unitSuffix", button);
	if ( unitSuffix ) then
		unit = unit .. unitSuffix;
		-- map raid1pet to raidpet1
		unit = gsub(unit, "^([^%d]+)([%d]+)[pP][eE][tT]$", "%1pet%2");
	end

	-- Don't do anything if our unit doesn't exist
	if ( unit and not UnitExists(unit) ) then
		return;
	end
 
	-- Remap button suffixes based on the disposition of the unit (contributed by Iriel and Cladhaire)
    if ( unit ) then
        if ( UnitCanAttack("player", unit) )then
            button = SecureButton_GetModifiedAttribute(self, "harmbutton", button) or button;
        elseif ( UnitCanAssist("player", unit) )then
            button = SecureButton_GetModifiedAttribute(self, "helpbutton", button) or button;
        end
    end

	-- Lookup the action type, based on the modifiers and button
	local type = SecureButton_GetModifiedAttribute(self, "type", button);

	-- Perform the requested action!
	if ( type == "action" ) then
		local action = SecureButton_GetModifiedAttribute(self, "action", button);
		if ( action ) then
			-- Save macros in case the one for this action is being edited
			securecall("MacroFrame_SaveMacro");

			UseAction(action, 0, unit);
		end
	elseif ( type == "pet" ) then
		local action = SecureButton_GetModifiedAttribute(self, "action", button);
		if ( action ) then
			CastPetAction(action, unit);
		end
	elseif ( type == "spell" ) then
		local spell = SecureButton_GetModifiedAttribute(self, "spell", button);
		if ( spell ) then
			CastSpellByName(spell, unit);
		end
	elseif ( type == "item" ) then
		local item = SecureButton_GetModifiedAttribute(self, "item", button);
		if ( item ) then
			UseItemByName(item, unit);
		end
	elseif ( type == "inventory" ) then
		local bag = SecureButton_GetModifiedAttribute(self, "bag", button);
		local slot = SecureButton_GetModifiedAttribute(self, "slot", button);
		if ( slot ) then
			if ( bag ) then
				UseContainerItem(bag, slot);
			else
				UseInventoryItem(slot);
			end
		end
	elseif ( type == "stop" ) then
		if ( SpellIsTargeting() ) then
			SpellStopTargeting();
		end
	elseif ( type == "target" ) then
		if ( SpellIsTargeting() ) then
			SpellTargetUnit(unit);
		elseif ( CursorHasItem() ) then
			DropItemOnUnit(unit);
		else
			TargetUnit(unit);
		end
	elseif ( type == "focus" ) then
		FocusUnit(unit);
	elseif ( type == "assist" ) then
		AssistUnit(unit);
	elseif ( type == "menu" ) then
		if ( self.showmenu ) then
			self:showmenu(unit);
		end
	end
end

function SecureUnitButton_OnLoad(self, unit, menufunc)
	self:SetAttribute("type1", "target");
	self:SetAttribute("type*", "menu");
	self:SetAttribute("unit", unit);
	self.showmenu = menufunc;
end

function SecureUnitButton_OnClick(self, button)
    local type = SecureButton_GetModifiedAttribute(self, "type", button);
    if ( type == "menu" ) then
        if ( SpellIsTargeting() ) then
            SpellStopTargeting();
            return;
        end
    end
    SecureActionButton_OnClick(self, button);
end

-- SecureStanceActions shows and hides its children based on their stance attribute:
--
--  0 = no special stance or form
--  1 = stance or form number 1
--  2 = stance or form number 2, etc.

function SecureStanceActions_OnLoad(self)
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORMS");
	self:RegisterEvent("PLAYER_AURAS_CHANGED");

	SecureStanceActions_UpdateState(self);
end

function SecureStanceActions_UpdateChildren(self, stance, ...)
    for i = 1, select("#",...) do
        local child = select(i,...);
		local childStance = child:GetAttribute("stance");
		if ( childStance ) then
			if ( childStance == stance ) then
				child:Show();
			else
				child:Hide();
			end
		end
    end
end

function SecureStanceActions_UpdateState(self)
	local stance = 0;

	for i = 1, GetNumShapeshiftForms() do
		local _,_,isActive = GetShapeshiftFormInfo(i);
		if ( isActive ) then
			stance = i;
			break;
		end
	end
	SecureStanceActions_UpdateChildren(self, stance, self:GetChildren());
end

--
-- SecureMenuCheckButton contributed with permission by: Tem
--

-- Secure Template for managing button placement and visibility.
-- Usage: 
--      local menuButton = CreateFrame("CheckButton","MenuButton1",UIParent,"SecureMenuCheckButtonTemplate")
--      
--      Any frame that it needs to manage should be set as a child of that button.
--
--      Attributes:
--          timeout: [number]
--              The amount of time in seconds after the menuButton or any of its children loses the mouse focus
--              that should be waited before hiding them again.
--
--          movetocursor: [boolean]
--              When set, the menuButton will be anchored directly below the mouse on showing its children.
--

function SecureMenuCheckButton_Update(self, shown, ...)
    for i = 1, select("#",...) do
        local child = select(i,...);
        if ( shown ) then
            child:Show();
        else
            child:Hide();
        end
    end
    self:SetChecked(shown)
end
      
function SecureMenuCheckButton_OnClick(self, button)
    local checked = self:GetChecked();
    SecureMenuCheckButton_Update(self, checked, self:GetChildren());
    if ( checked ) then
        self.timeleft = self:GetAttribute("timeout");
        if( self:GetAttribute("movetocursor") and GetMouseFocus() ~= self ) then
            local s,x,y = self:GetEffectiveScale(),GetCursorPosition();
            self:SetPoint("CENTER",UIParent,"BOTTOMLEFT",x/s,y/s)            
        end
    else
        self.timeleft = nil;
    end
end

function SecureMenuCheckButton_MouseHovering(self, ...)
    local mouseFocus = GetMouseFocus();
    if ( not mouseFocus ) then
        return false;
    end
    if ( mouseFocus == self ) then
        return true;
    end
    for i = 1, select("#",...) do
        local child = select(i,...)
        if ( mouseFocus == child ) then
            return true;
        end
    end
    return false;
end

function SecureMenuCheckButton_OnUpdate(self, elapsed)
    if ( self.timeleft ) then
        if ( SecureMenuCheckButton_MouseHovering(self, self:GetChildren()) ) then
            self.timeleft = self:GetAttribute("timeout");
            return;
        end
        local timeleft = self.timeleft - elapsed;
        if ( timeleft <= 0 ) then
            SecureMenuCheckButton_Update(self, false, self:GetChildren());
            self.timeleft = nil;
        else
            self.timeleft = timeleft;
        end
    end
end

--
-- SecureRaidGroupHeader contributed with permission by: Esamynn, Cide, and Iriel
--

--[[
List of the various configuration attributes
======================================================
nameList = [STRING] -- a comma separated list of player names (not used if 'groupFilter' is set)
groupFilter = [1-8, STRING] -- a raid group number or an uppercase class name
point = [STRING] -- a valid XML anchoring point (Default: "TOP")
xOffset = [NUMBER] -- the x-Offset to use when anchoring the unit buttons (Default: 0)
yOffset = [NUMBER] -- the y-Offset to use when anchoring the unit buttons (Default: 0)
sortMethod = ["INDEX", "NAME"] -- defines how the group is sorted (Default: "INDEX")
sortDir = ["ASC", "DESC"] -- defines the sort order (Default: "ASC")
template = [STRING] -- the XML template to use for the unit buttons
======================================================
--]]

function SecureRaidGroupHeader_OnLoad(self)
	self:RegisterEvent("RAID_ROSTER_UPDATE");
	this.buttons = {};
end

function SecureRaidGroupHeader_OnEvent(self, event)
	if ( event == "RAID_ROSTER_UPDATE" and self:IsVisible() ) then
		SecureRaidGroupHeader_Update(self);
	end
end

-- relativePoint, xMultiplier, yMultiplier = getRelativePointAnchor( point )
-- Given a point return the opposite point and which axes the point
-- depends on.
local function getRelativePointAnchor( point )
	point = strupper(point);
	if (point == "TOP") then
		return "BOTTOM", 0, -1;
	elseif (point == "BOTTOM") then
		return "TOP", 0, 1;
	elseif (point == "LEFT") then
		return "RIGHT", 1, 0;
	elseif (point == "RIGHT") then
		return "LEFT", -1, 0;
	elseif (point == "TOPLEFT") then
		return "BOTTOMRIGHT", 1, -1;
	elseif (point == "TOPRIGHT") then
		return "BOTTOMLEFT", -1, -1;
	elseif (point == "BOTTOMLEFT") then
		return "TOPRIGHT", 1, 1;
	elseif (point == "BOTTOMRIGHT") then
		return "TOPLEFT", -1, 1;
	else
		return "CENTER", 0, 0;
	end
end

local sortingTable = {}; --working table

function SecureRaidGroupHeader_Update(self)
	local nameList = self:GetAttribute("nameList");
	local groupFilter = self:GetAttribute("groupFilter");
	
	if not ( groupFilter or nameList ) then
		self:Hide();
		return;
	end
	
	local point = self:GetAttribute("point") or "TOP"; --default anchor point of "TOP"
	local relativePoint, xOffsetMult, yOffsetMult = getRelativePointAnchor(point);
	local xMultiplier, yMultiplier =  abs(xOffsetMult), abs(yOffsetMult);
	local xOffset = self:GetAttribute("xOffset") or 0; --default of 0
	local yOffset = self:GetAttribute("yOffset") or 0; --default of 0
	local sortMethod = self:GetAttribute("sortMethod") or "INDEX"; --sort by ID by default
	local sortDir = self:GetAttribute("sortDir") or "ASC"; --sort ascending by default
	
	for k, v in pairs(sortingTable) do
		sortingTable[k] = nil;
	end
	
	if ( type(groupFilter) == "number" ) then
		-- sorting by group number
		for i = 1, GetNumRaidMembers(), 1 do
			local name, _, subgroup = GetRaidRosterInfo(i);
			if ( groupFilter == subgroup ) then
				tinsert(sortingTable, name);
				sortingTable[name] = "raid"..i;
			end
		end
	
	elseif ( type(groupFilter) == "string" ) then
		-- sorting by class
		for i = 1, GetNumRaidMembers(), 1 do
			local name, _, subgroup, _, _, className = GetRaidRosterInfo(i);
			if ( groupFilter == className ) then
				tinsert(sortingTable, name);
				sortingTable[name] = "raid"..i;
			end
		end
	
	elseif ( type(nameList) == "string" ) then
		for name in string.gmatch(nameList, "[%w]+") do
			tinsert(sortingTable, name);
			sortingTable[name] = true;
		end
		for i = 1, GetNumRaidMembers(), 1 do
			local name = GetRaidRosterInfo(i);
			if ( sortingTable[name] ) then
				sortingTable[name] = "raid"..i;
			end
		end
		for i = #sortingTable, 1, -1 do
			local name = sortingTable[i];
			if ( sortingTable[name] == true ) then
				tremove(sortingTable, i);
			end
		end
	
	else
		self:Hide();
		return;
	end
	
	if ( sortMethod == "NAME" ) then
		table.sort(sortingTable);
	end
	
	local unitCount = #sortingTable;
	
	-- ensure there are enough buttons
	local buttons = this.buttons;
	local needButtons = max(1, unitCount);
	if ( #buttons < needButtons ) then
		local buttonTemplate = self:GetAttribute("template");
		local name = self:GetName();
		if not ( buttonTemplate and name ) then
			self:Hide();
			return;
		end
		for i = (#buttons + 1), needButtons, 1 do
			buttons[i] = CreateFrame("Button", name.."UnitButton"..i, this, buttonTemplate);
		end
	end
	
	local loopStart, loopFinish, step = 1, unitCount, 1;
	if ( sortDir == "DESC" ) then
		loopStart, loopFinish, step = unitCount, 1, -1;
	end
	
	local buttonNum = 0;
	local currentAnchor = this;
	for i = loopStart, loopFinish, step do
		buttonNum = buttonNum + 1;
		
		local unitButton = buttons[buttonNum];
		unitButton:Hide();
		unitButton:ClearAllPoints();
		if ( buttonNum == 1 ) then
			unitButton:SetPoint(point, currentAnchor, point, 0, 0);
		else
			unitButton:SetPoint(point, currentAnchor, relativePoint, xMultiplier * xOffset, yMultiplier * yOffset);
		end
		unitButton:SetAttribute("unit", sortingTable[sortingTable[i]]);
		unitButton:Show();
		
		currentAnchor = unitButton;
	end
	for i = (buttonNum + 1), #buttons, 1 do
		buttons[i]:Hide();
		buttons[i]:SetAttribute("unit", nil);
	end
	
	local unitButtonWidth = buttons[1]:GetWidth();
	local unitButtonHeight = buttons[1]:GetHeight();
	if ( unitCount > 0 ) then
		self:SetWidth( xMultiplier * (unitCount - 1) * unitButtonWidth + ( (unitCount - 1) * (xOffset * xOffsetMult) ) + unitButtonWidth );
		self:SetHeight( yMultiplier * (unitCount - 1) * unitButtonHeight + ( (unitCount - 1) * (yOffset * yOffsetMult) ) + unitButtonHeight );
	else
		local minWidth = self:GetAttribute("minWidth") or (yMultiplier * unitButtonWidth);
		local minHeight = self:GetAttribute("minHeight") or (xMultiplier * unitButtonHeight);
		self:SetWidth( max(minWidth, 0.1) );
		self:SetHeight( max(minHeight, 0.1) );
	end
end
